@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.6 2006/01/03 03:33:10 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.8 2006/02/14 08:19:06 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2005	Tatsuo Ishii
+ * Copyright (c) 2003-2006	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -81,7 +81,7 @@ void do_child(int unix_fd, int inet_fd)
 	struct timeval timeout;
 	static int connected;
 	int connections_count = 0;	/* used if child_max_connections > 0 */
-
+	int first_ready_for_query_received;		/* for master/slave mode */
 
 	pool_debug("I am %d", getpid());
 
@@ -217,6 +217,8 @@ void do_child(int unix_fd, int inet_fd)
 		 * we need to connect to the backend and send the startup packet.
 		 */
 
+		first_ready_for_query_received = 0;		/* for master/slave mode */
+
 		/* look for existing connection */
 		if ((backend = pool_get_cp(sp->user, sp->database, sp->major)) == NULL)
 		{
@@ -225,7 +227,15 @@ void do_child(int unix_fd, int inet_fd)
 
 			if ((backend = connect_backend(sp, frontend)) == NULL)
 				continue;
+
+			/* in master/slave mode, the first "ready for query"
+			 * packet should be treated as if we were not in the
+			 * mode
+			 */
+			if (MASTER_SLAVE)
+				first_ready_for_query_received = 1;
 		}
+
 		else
 		{
 			/* we don't need Startup Packet info anymore */
@@ -281,7 +291,7 @@ void do_child(int unix_fd, int inet_fd)
 		{
 			POOL_STATUS status;
 
-			status = pool_process_query(frontend, backend, 0);
+			status = pool_process_query(frontend, backend, 0, first_ready_for_query_received);
 
 			sp = MASTER_CONNECTION(backend)->sp;
 
@@ -308,7 +318,7 @@ void do_child(int unix_fd, int inet_fd)
 						POOL_STATUS status1;
 
 						/* send reset request to backend */
-						status1 = pool_process_query(frontend, backend, 1);
+						status1 = pool_process_query(frontend, backend, 1, 0);
 						pool_close(frontend);
 
 						/* if we detect errors on resetting connection, we need to discard
