@@ -1,7 +1,7 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool/pool.h,v 1.7 2006/02/05 01:59:27 devrim Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool.h,v 1.8 2006/02/14 08:19:06 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -266,7 +266,8 @@ extern void do_child(int unix_fd, int inet_fd);
 extern int pool_init_cp(void);
 extern POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 									  POOL_CONNECTION_POOL *backend,
-									  int connection_reuse);
+									  int connection_reuse,
+									  int first_ready_for_query_received);
 
 extern POOL_CONNECTION *pool_open(int fd);
 extern void pool_close(POOL_CONNECTION *cp);
