@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.12 2006/02/05 01:59:27 devrim Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.13 2006/02/14 08:19:06 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -119,7 +119,8 @@ static int is_drop_database(char *query);		/* returns non 0 if this is a DROP DA
 
 POOL_STATUS pool_process_query(POOL_CONNECTION *frontend, 
 							   POOL_CONNECTION_POOL *backend,
-							   int connection_reuse)
+							   int connection_reuse,
+							   int first_ready_for_query_received)
 {
 	char kind, kind1;	/* packet kind (backend) */
 	char fkind;	/* packet kind (frontend) */
@@ -315,7 +316,7 @@ POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 		}
 
 		/* this is the synchronous point */
-		if (REPLICATION)
+		if (REPLICATION || first_ready_for_query_received)
 		{
 			if (kind == 0)
 			{
@@ -349,6 +350,8 @@ POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 				}
 			}
 
+			first_ready_for_query_received = 0;
+
 			if (kind == '\0' || kind != kind1)
 			{
 				int sts;
@@ -1660,6 +1663,13 @@ static POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 		default:
 			if (MAJOR(backend) == PROTO_MAJOR_V3)
 			{
+				if (MASTER_SLAVE)
+				{
+					master_slave_was_enabled = 1;
+					MASTER_SLAVE = 0;
+					master_slave_dml = 1;
+				}
+
 				status = SimpleForwardToBackend(fkind, frontend, backend);
 				if (pool_flush(MASTER(backend)))
 					status = POOL_ERROR;
